/**************************************************************************
 *   Copyright (C) 2009 by Peter Semiletov                                 *
 *   peter.semiletov@gmail.com                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui>
#include <QtNetwork>

#include <phonon/phononnamespace.h>
#include <phonon/audiooutput.h>
#include <phonon/seekslider.h>
#include <phonon/mediaobject.h>
#include <phonon/volumeslider.h>
#include <phonon/backendcapabilities.h>
#include <phonon/effect.h>
#include <phonon/effectwidget.h>
#include <phonon/effectparameter.h>


#include <fman.h>
#include "img_viewer.h"
#include "metainf.h"


class CLogMemo: public QObject
{
  Q_OBJECT

public:

  QPlainTextEdit *memo;
  CLogMemo (QPlainTextEdit *m);

public slots:

  void log (const QString &text);
};


class CAboutWindow: public QWidget
{
  Q_OBJECT

  QLabel *logo;
  
public:

  CAboutWindow();

protected:

  void closeEvent (QCloseEvent *event);
};


class CFxWindow: public QWidget
{
  Q_OBJECT

public:

  QListWidget *list;
  
  CFxWindow();

protected:

  void closeEvent (QCloseEvent *event);
};


class CReloadWindow: public QWidget
{
  Q_OBJECT

public:

  QListWidget *list;
  
  CReloadWindow (const QString &def);

protected:

  void closeEvent (QCloseEvent *event);
};


class MainWindow : public QMainWindow
{
  Q_OBJECT
  
public:

  QHttp http_lyrics;

  QList<Phonon::EffectDescription> effectDescriptions;
  QListWidget *lw_fx;
  
  Phonon::SeekSlider *seek_slider;
  Phonon::MediaObject *mediaObject;
  Phonon::AudioOutput *audioOutput;
  Phonon::VolumeSlider *slider_volume;
  Phonon::Path sound_path;
  
  QComboBox *cb_fman_drives;
  
  QNetworkAccessManager lyr_nmanager;
  
  QHash <QString, QString> mime_types;  
  QHash <QString, QString> mime_types_supported;  
    
  QStringList sl_places_bmx; 
  QStringList sl_fx_names;   
  
  QString tags_format;
  QString charset;
  QString prev_charset;
  
  QString fname_places_bookmarks;
  QString fname_fx;

  QString dir_config;
  QString dir_fx_presets;

  bool ui_update;
  bool prefere_idv1;

  Cid3v1 media_fields_idv1;
  Cid3v2 media_fields_idv2;
  
  CLogMemo *log;  
  CReloadWindow *enc_window;
  CFxWindow *fx_window;

  CFMan *fman;
  CImgViewer *img_viewer; 

  QSettings *settings;
  QTranslator myappTranslator;
  QTranslator qtTranslator;

  QSystemTrayIcon *ico_tray;
  
  QLabel *l_time;
  QLabel *l_total_time;
    
  QVBoxLayout *lv_player_layout;
  QToolBar *tb_volume;

  QTabWidget *main_tab_widget;
  
  QWidget *ctl_player;
  QWidget *wd_fman;
  QLineEdit *ed_fman_path;

  QToolBar *tb_fman_dir;
  QToolBar *tb_player;
  
  QWidget *w_right;
  QListWidget *lv_places;
  QSplitter *spl_fman;
  
  QPlainTextEdit *ple_memo;
  QTextBrowser *man;  
  QTextBrowser *text_file_browser;

  QSplitter *spl_main;

  QMenu *fileMenu;
  QMenu *helpMenu;
  QMenu *viewMenu;
  QMenu *funcsMenu;
  QMenu *view_themes_menu;
  QMenu *funcs_fileops;

  QMenu *fx_presets;

  
  QAction *act_test; 
  
  QAction *act_play; 
  QAction *act_stop; 
  QAction *act_pause; 
  QAction *act_next; 
  QAction *act_prev; 

  QAction *act_exit;
  QAction *act_about_qt;
  
  
  MainWindow (QWidget *parent = 0);
  ~MainWindow();
       
  
  QAction* add_to_menu (QMenu *menu,
                        const QString &caption,
                        const char *method,
                        const QString &shortkt = QString(),
                        const QString &iconpath = QString()
                        );

  
  void create_fman();
  void create_ctl_player();
  
  void readSettings();
  void writeSettings();
  void create_paths();
  void create_actions();
  void create_menus();
  void create_manual();
  void create_tune();

  void enq_next();
  void update_places_bookmarks();
  void update_themes_menu();
  void update_fx_presets_menu();

  
  void setup_mime_table();
  void show_text_file (const QString &fname);
  void show_html_data (const QString &data);

  void show_text_file_enc (const QString &fname);

  bool is_file_supported_by_mime (const QString &fname);

  void parse_current_tags();
  void handle_args();
  void read_fx (const QString &fname);
  void fx_save (const QString &fname);


  Phonon::EffectDescription get_fx_desc_by_name (const QString &name);
  Phonon::EffectParameter get_param_by_name (Phonon::Effect *effect, const QString &name);

  
public slots:

  void hard_next();
  void hard_prev();
  void play();
  void stop();
  
  void supported_mimes();
  void unknown_mimes();
  
  void test();

  void tick (qint64 time);
  void sourceChanged (const Phonon::MediaSource &source);
  void totalTimeChanged (qint64 newTotalTime); 
  void metaDataChanged();
  void aboutToFinish();  

  void prefere_idv1_state_changed (int state);

  void select_prev_enc();

  
  void fman_rename();
  void fman_delete();

  
  void fman_add_bmk();
  void fman_del_bmk();
  void fman_places_itemActivated (QListWidgetItem *item);
  void fman_dir_changed (const QString &full_path);
  void fman_naventry_confirm();
  void fman_refresh_dir();
  void fman_drives_changed (const QString & path);
  void fman_file_activated (const QString &full_path);

  void use_theme_triggered();
  void tray_activated (QSystemTrayIcon::ActivationReason reason);
  void fn_get_file_info();

  void show_changelog();
  void show_gpl();
  void show_news();
  void about();
   
  void cb_outdevs_currentIndexChanged (int index);
  //void override_out_state_changed (int state);
  void cb_info_in_tray_state_changed (int state);
  void cb_dont_play_next_file_state_changed (int state);

  
  void enc_itemDoubleClicked (QListWidgetItem *item);
  void select_enc();
  
  void replyFinished (QNetworkReply* r);
  void view_lyr();

  
  void fx_add();
  void fx_del();
  void fx_save();
  
  void fx_itemDoubleClicked (QListWidgetItem *item);
  void fx_box_itemDoubleClicked (QListWidgetItem *item);
  void lw_fx_currentItemChanged (QListWidgetItem *current, QListWidgetItem *previous); 
  
  void use_fx_preset();
};

#endif // MAINWINDOW_H
